/**
 * Copyright (C) 2007 by Levi Bard
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * @author Levi Bard <taktaktaktaktaktaktaktaktaktak@gmail.com>
 */

#include "ruby-lightmediascanner.h"
#include "lightmediascanner_scanner.h"

/**
 * Scanner#new
 *
 * constructor
 *
 * * p1 (db_path) The path to the sqlite3 db to be used
 *
 * * return A new Scanner instance
 */
static VALUE scanner_new(VALUE obj, VALUE db_path) {
	Check_Type(db_path, T_STRING);
	return Data_Wrap_Struct(lightmediascanner_scanner, 0, lms_free, lms_new(StringValuePtr(db_path)));
}// scanner_new

/**
 *
 * Scanner::process
 *
 * This will add or update media found in the given directory or its children.
 *
 * * p1 (top_path) The top directory to scan.
 *
 * * return 0 on success
 *
 */
static VALUE scanner_process(VALUE self, VALUE top_path) {
	lms_t *lms;
	int rv = 0;
	char *ctop_path = NULL;

	Check_Type(top_path, T_STRING);
	Data_Get_Struct(self, lms_t, lms);
	ctop_path = StringValuePtr(top_path);

	rv = lms_process(lms, ctop_path);

	if(rv) {
		rb_raise(rb_eRuntimeError, "Failure processing %s", ctop_path);
	}// FAIL

	return INT2FIX(lms_process(lms, StringValuePtr(top_path)));
}// scanner_process

/**
 *
 * Scanner::check
 *
 * Check consistency of the given directory.
 *
 * * p1 (top_path) The top directory to scan.
 *
 * * return 0 on success
 *
 */
static VALUE scanner_check(VALUE self, VALUE top_path) {
	lms_t *lms;
	int rv = 0;
	char *ctop_path = NULL;

	Check_Type(top_path, T_STRING);
	Data_Get_Struct(self, lms_t, lms);
	ctop_path = StringValuePtr(top_path);

	rv = lms_process(lms, ctop_path);

	if(rv) {
		rb_raise(rb_eRuntimeError, "Failure checking %s", ctop_path);
	}// FAIL

	return INT2FIX(lms_check(lms, StringValuePtr(top_path)));
}// scanner_check

/**
 *
 * Scanner::db_path
 *
 * Accessor for LMS DB path.
 *
 * * return The database path given at creation time
 *
 */
static VALUE scanner_db_path(VALUE self) {
	lms_t *lms;
	Data_Get_Struct(self, lms_t, lms);

	return rb_tainted_str_new2(lms_get_db_path(lms));
}// scanner_db_path

/**
 *
 * Scanner::processing?
 *
 * Checks if Light Media Scanner is being used in a processing operation like lms_process() or lms_check().
 *
 * * return Boolean representing the instance's processing state
 *
 */
static VALUE scanner_processing(VALUE self) {
	lms_t *lms;
	Data_Get_Struct(self, lms_t, lms);

	return (lms_is_processing(lms)? Qtrue: Qfalse);
}// scanner_processing

/**
 *
 * Scanner::timeout
 *
 * Get the maximum amount of milliseconds the slave can take to serve one file. 
 *
 * If a slave takes more than this amount of milliseconds, it will be killed and the scanner will continue with the next file.
 *
 * * return The timeout in milliseconds, or -1 on error
 *
 */
static VALUE scanner_get_slave_timeout(VALUE self) {
	lms_t *lms;
	Data_Get_Struct(self, lms_t, lms);

	return INT2FIX(lms_get_slave_timeout(lms));
}// scanner_get_slave_timeout

/**
 *
 * Scanner::timeout=
 *
 * Set the maximum amount of milliseconds the slave can take to serve one file.
 *
 * If a slave takes more than this amount of milliseconds, it will be killed and the scanner will continue with the next file.
 *
 * * p1 (timeout_ms) The timeout in milliseconds
 *
 * * return The new timeout, in milliseconds
 *
 */
static VALUE scanner_set_slave_timeout(VALUE self, VALUE timeout_ms) {
	lms_t *lms;

	Check_Type(timeout_ms, T_FIXNUM);
	Data_Get_Struct(self, lms_t, lms);

	lms_set_slave_timeout(lms, FIX2INT(timeout_ms));
	return INT2FIX(lms_get_slave_timeout(lms));
}// scanner_set_slave_timeout

/**
 *
 * Scanner::commit_interval
 *
 * Get the number of files served between database transactions.
 *
 * * return The number of files served between database transactions
 *
 */
static VALUE scanner_get_commit_interval(VALUE self) {
	lms_t *lms;
	Data_Get_Struct(self, lms_t, lms);

	return INT2FIX(lms_get_commit_interval(lms));
}// scanner_get_commit_interval

/**
 *
 * Scanner::commit_interval=
 *
 * Set the number of files to be served between database transactions.
 *
 * * p1 (transactions) The number of files between commits
 *
 * * return The new number of files to be served between database transactions
 *
 */
static VALUE scanner_set_commit_interval(VALUE self, VALUE transactions) {
	lms_t *lms;

	Check_Type(transactions, T_FIXNUM);
	Data_Get_Struct(self, lms_t, lms);

	lms_set_commit_interval(lms, FIX2INT(transactions));
	return INT2FIX(lms_get_commit_interval(lms));
}// scanner_set_commit_interval

/**
 *
 * Scanner::add_charset
 *
 * Register a new charset encoding to be used.
 *
 * Throws a RuntimeError if LMS is unable to add the charset.
 *
 * * p1 (charset) charset name as understood by iconv_open(3).
 *
 * * return charset
 *
 */
static VALUE scanner_add_charset(VALUE self, VALUE charset) {
	lms_t *lms;
	char *ccharset = NULL;

	Check_Type(charset, T_STRING);
	Data_Get_Struct(self, lms_t, lms);
	ccharset = StringValuePtr(charset);

	if(lms_charset_add(lms, ccharset)) {
		rb_raise(rb_eRuntimeError, "Unable to add charset %s", ccharset);
	}//FAIL

	return charset;
}// scanner_add_charset

/**
 *
 * Scanner::remove_charset
 *
 * Forget about a registered charset encoding.
 *
 * Throws a RuntimeError if LMS is unable to remove the charset.
 *
 * * p1 (charset) charset name as understood by iconv_open(3).
 *
 * * return charset
 *
 */
static VALUE scanner_remove_charset(VALUE self, VALUE charset) {
	lms_t *lms;
	char *ccharset = NULL;

	Check_Type(charset, T_STRING);
	Data_Get_Struct(self, lms_t, lms);
	ccharset = StringValuePtr(charset);
	if(lms_charset_del(lms, ccharset)) {
		rb_raise(rb_eRuntimeError, "Unable to remove charset %s", ccharset);
	}//FAIL

	return charset;
}// scanner_remove_charset

/**
 * LightMediaScanner::Scanner is a class to represent a 
 * LightMediaScanner instance.
 */
void Init_lightmediascanner_scanner() {
	lightmediascanner_scanner = rb_define_class_under(lightmediascanner_module, "Scanner", rb_cObject);
	rb_define_singleton_method(lightmediascanner_scanner, "new", scanner_new, 1);
	rb_define_method(lightmediascanner_scanner, "process", scanner_process, 1);
	rb_define_method(lightmediascanner_scanner, "check", scanner_check, 1);
	rb_define_method(lightmediascanner_scanner, "db_path", scanner_db_path, 0);
	rb_define_method(lightmediascanner_scanner, "processing?", scanner_processing, 0);
	rb_define_method(lightmediascanner_scanner, "timeout", scanner_get_slave_timeout, 0);
	rb_define_method(lightmediascanner_scanner, "timeout=", scanner_set_slave_timeout, 1);
	rb_define_method(lightmediascanner_scanner, "commit_interval", scanner_get_commit_interval, 0);
	rb_define_method(lightmediascanner_scanner, "commit_interval=", scanner_set_commit_interval, 1);
	rb_define_method(lightmediascanner_scanner, "add_charset", scanner_add_charset, 1);
	rb_define_method(lightmediascanner_scanner, "remove_charset", scanner_remove_charset, 1);
}// Init_lightmediascanner
